// explanatory note:
//   DFJan1 = Day From Jan1

#define   PROGRAM_NAME       "Julian date converter"
#define   PROGRAM_DOSNAME    "JulDate"
#define   PROGRAM_VERSION    "2.0"
#define   PROGRAM_DATE       __DATE__


#include  <string.h>
#include  <stdlib.h>
#include  <stdio.h>


typedef  int  boolean;
#define     FALSE   (0)
#define     TRUE    (!FALSE)




boolean  is_leap(int y)
{
  if ((y % 400) == 0)                    /* All years divisible by */
    return TRUE;                         /*  400 are leap years */
  if ((y % 100) == 0)                    /* Other years divisible by */
    return FALSE;                        /*  100 are not leap years */
  return ((y % 4) == 0);                 /* Other years divisible by */
}                                        /*  4 are leap years */




int dim(int m,int y)
/*  returns the number of days in the current month */
{
  char    mdt[] = { 0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };

  if ((m == 2) && is_leap(y))                 /* February in leap year */
    return 29;
  else                                        /* Normal month */
    return mdt[m];
} /* End dim() */




void bump_month(int *m,int *,int *y)
{
  if (*m == 12)                   /* If it's December, */
  {
    *m = 1;                       /*  roll 'round to January */
    *y += 1;                      /*  of the next year */
  }                               /* End "it's December" */
  else                            /* Otherwise, just bump the */
    *m += 1;                      /*  month */
}  /* End bump_month */


void bump_day(int *m,int *d,int *y)
{
  if (*d == dim(*m,*y))           /* If at end of month, */
  {
    *d = 1;                       /*  roll 'round to start of */
    bump_month(m,d,y);            /*  the next month */
  }                               /* End 'end of month' */
  else                            /* Otherwise, just bump the */
    *d += 1;                      /*  day */
}  /* End bump_day */




int  DayFromJan1(int m, int d, int y)
/*
  This routine calculates the number of days from January 1 of the
  year indicated.  Note that January 1 = 1.  This routine is intended
  to calculate Julian dates.
*/
{
  long dfj1;
  int  Ndays[13] =       /* Number of days before the first of the month */
         { 0, 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334};


  if (m > 0)
    dfj1 = Ndays[m];    /* start off with days for the months passed by */

  if (m > 2)
    dfj1 += (is_leap(y) ? 1 : 0);   /* add an extra day in leap years */

  dfj1 += d;            /* add in the number of days */

  return (dfj1);
}  /* DayFromJan1 */




void DFJan1_to_date(int DFJan1, int *m, int *d, int *y)
/*
  This routine converts an offset day from Jan 1 to a m,d,y, the
  components of a date.
*/
{
  *d = DFJan1;
  for (*m = 1; *d > dim(*m,*y); )
  {
    *d -= dim(*m,*y);
    bump_month(m,d,y);
  }

  return;
}




main(int argc, char *argv[])
{
  int      DFJan1,
           M,
           D,
           Y;
  char     orig_char;
  boolean  args_ok = FALSE;


  /* say hello */
  printf("%s - %s  Version %s - %s\n", PROGRAM_DOSNAME, PROGRAM_NAME,
                                     PROGRAM_VERSION, PROGRAM_DATE );

  switch (strlen(argv[1]))
  {
  case 5:
    /* convert yyjjj to mm/dd/yy */
    /* extract year */
    orig_char = argv[1][2];     // save original char
    argv[1][2] = 0;             // zero terminate year
    Y = atoi(argv[1]);          // convert
    argv[1][2] = orig_char;     // put back original character

    DFJan1 = atoi(argv[1]+2);             // extract DFJan1

    DFJan1_to_date(DFJan1, &M, &D, &Y);   // get complete calendar date
    args_ok = TRUE;
    break;
  case 6:
    /* convert mmddyy to yyjjj */
    /* extract mm */
    orig_char = argv[1][2];     // save original char
    argv[1][2] = 0;             // zero terminate year
    M = atoi(argv[1]+0);        // convert
    argv[1][2] = orig_char;     // put back original character
    /* extract dd */
    orig_char = argv[1][4];     // save original char
    argv[1][4] = 0;             // zero terminate year
    D = atoi(argv[1]+2);        // convert
    argv[1][4] = orig_char;     // put back original character
    /* extract yy */
    // not necessary to zero terminate year -
    //   already done since year is at end of string
    Y = atoi(argv[1]+4);        // convert

    DFJan1 = DayFromJan1(M, D, Y);
    args_ok = TRUE;
    break;
  case 8:
    /* convert mm/dd/yy to yyjjj */
    /* convert mmddyy to yyjjj */
    /* extract mm */
    orig_char = argv[1][2];     // save original char
    argv[1][2] = 0;             // zero terminate year
    M = atoi(argv[1]+0);        // convert
    argv[1][2] = orig_char;     // put back original character
    /* extract dd */
    orig_char = argv[1][5];     // save original char
    argv[1][5] = 0;             // zero terminate year
    D = atoi(argv[1]+3);        // convert
    argv[1][5] = orig_char;     // put back original character
    /* extract yy */
    // not necessary to zero terminate year -
    //   already done since year is at end of string
    Y = atoi(argv[1]+6);        // convert

    DFJan1 = DayFromJan1(M, D, Y);
    args_ok = TRUE;
    break;
  default:
    printf("Unrecognizable argument given.  Use yyjjj, mmddyy or mm/dd/yy.\n");
    args_ok = FALSE;
    break;
  }

  if (args_ok)
    printf("Julian date = %02d%03d    Calendar date = %02d/%02d/%02d\n", Y, DFJan1, M, D, Y);

  return 0;
}
